% Spatial Modulation   
% Reproduce Paper "Spatial Modulation: Optimal Detection and Performance
% Analysis"
% Copyright @ Dr. Samir Al-Ghadhban {samir@kfupm.edu.sa)
% Last modified: 23/11/2008

clear all; 
% Start
format short e

% disp('choose either:')
% disp('1- Frame Error Rate simulation')
% disp('2- Bit Error Rate simulation')
% SimType=input('Input your selection >> ');
SimType=2; 

disp('Input modulation type')
disp(' 1- BPSK')
disp(' 2- QPSK')
disp(' 3- 8PSK')
disp(' 4- 16QAM')
disp(' 5- 64QAM')
disp(' 6- 256QAM')

Modulation=input('Input your selection >> ');
[SignalSet,BitMapping_q,S,Es,MType]=choosesignalset(Modulation);



% disp('Select the ordering method')
% disp('1- No Ordering')
% disp('2- ZF with ordering')
% disp('3- MMSE with ordering')
% ordering=input('Input your selection >> ');
            
Trail=input('Input max number of errors counted in the simulation >> ');
NumberOfBits=input('Input minumum number of frames or bits >> ');
NTx=input('Input  number of Transmit antennas >> ');
MRx=input('Input number of receive antennas  >> ');
SNR=input('Input SNR range in dB  >> ');

BitMapping_j=dec2bin(0:NTx-1,2); % Bitmapping for each transmit antenna

qset=length(SignalSet); 
Bits=0;

effq=log2(qset); % Bits per signal set
effj=log2(NTx);  % Bits per antenna

BER=zeros(1,length(SNR));
H=zeros(MRx,NTx);
AWGN=zeros(MRx,1);
Y=zeros(MRx,1);
% X=zeros(NTx,1);
% XInd=zeros(NTx,1);

for jj=1:length(SNR)
    ER=0;
    
    Bits=0;
    
    Sigma2=Es*10^(-SNR(jj)/10);  %% noise variance 
    
    while ((min(ER)<Trail)|((Bits<NumberOfBits)&(min(ER)>=Trail)))
      
        Indexq=ceil(qset*rand(1,1));   % Signal point
        Indexj=ceil(NTx*rand(1,1));    % Antenna  
        X=SignalSet(Indexq);               
        Bits=Bits+effq+effj;                          %% bit Counter
        
%     switch SimType
%     case 1
%     Bits=Bits+1; % the signals transmitted from all groups constitute one frame
%     case 2
%      Bits=Bits+NTx*log2(qset);                          %% bit Counter
%     end
%  
        
        %% The Fading Channel
        MType='complex';
        H=GenerateFade(MRx,NTx,MType);
     
        
        %The received vector
        
        %Generate the Gaussain Noise 
        AWGN=GenerateNoise(MRx,MType,Sigma2);
        
        Y=H(:,Indexj)*X+AWGN; % one transmission at a time 
       
        % Spatial Demodulator 

        ER=ER+SpatialDem(Y,H,SignalSet,Indexj,Indexq,BitMapping_j,BitMapping_q,SimType);

        
%         switch ordering
%         case 1
%         ER=ER+ZFBLASTNoOrder(Y,H,SignalSet,Modulation,RandomIndex,BitMapping,SimType);
%         case 2
%         ER=ER+ZFBLASTOrder(Y,H,SignalSet,Modulation,RandomIndex,BitMapping,SimType);
%         case 3
%         ER=ER+MMSEBLASTOrder(Y,H,SignalSet,Modulation,Sigma2,RandomIndex,BitMapping,SimType);
%         end
    
    end % end while loop
    BER(:,jj)=ER/Bits
    
end %% end SNR loop
figure;
semilogy(SNR,BER,'b');
xlabel('SNR in dB');

switch SimType
case 1
ylabel('FER');
case 2
ylabel('BER');
end

title(sprintf('Spatial Modulation performance at N_T=%2.0f M_R=%2.0f and at %2.0f bps/Hz  ',[NTx MRx effq+effj]));
% switch ordering
% case 1
% legend('ZF No Order');
% case 2
% legend('ZF with Ordering');
% case 3
% legend('MMSE with Ordering');
% end